package rocketBugsRender;
/*
	Env.java ... painting parameters only here
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.io.*;
import java.lang.Math.*;
import java.text.*;

public class Env {
	
	//**** BASIC ****
	static double deltaT = 0.0001;					// time step for the simulation... read from file
	static double curTime = 0;						// current time.. figured from file names
	static final Pt3D dimensions = new Pt3D(20,20,20);	// dimensions of cytosolic space x,y,z in m
	static final int maxPlayers = 8000;				// dimension for playerList
	//**** actin parameters ****
	static final double lengthPerActin = 0.0027;		// length added to filament by one actin monomer
	static final double fActinDiameter = 0.007;		// diameter of actin filament in m
	static final double helixWavelength = 0.037;	// length for helical repeat in actin filaments, m
	static final double actinPerTurn = helixWavelength/lengthPerActin; 	// number of monomers per helical turn
	static final double helixAngPerActin = Math.PI/actinPerTurn;	// helical turn per actin monomer...only 1/2 turn per wavelength

	//****
	static boolean paused = true;
	//**** LISTERIA PARAMETERS ****
	static double listeriaLength = 0.8;	// length of listeria
	static double listeriaCylDiam = 0.7; // diameter of listeria cylinder
	
	//**** ACTIN TRACKING *****
	static boolean trackActin = false;
	
	//**** PATH TRACKING *****
	static Pt3D [] pathPts = new Pt3D [10000];
	static int pathPtCt = 0;
	static boolean trackPath = true;
	static int trackSkipInt = 1;
	static boolean connectPts = true;
	static int pathCircleRad = 2;
	
	//**** BATCHING *****
	static boolean batching = false;		// batching flag
	static boolean [] dataColumnsOn;		// columns we are loading
	
	//**** ZOOMS and SUCH ******
	static double initialScale = 25.0;		// initial zoom

	//**** PAINTING VARIABLES ****
		// text formatting
	static DecimalFormat scaleBarFormat = new DecimalFormat("#00.00;#00.00#");
	static DecimalFormat timeFormat = new DecimalFormat("#000.0000;#000.0000#");
		// ***** window sizing ***
	static int zoomWindowSize = 600;					// size of zoomPanel
	static double scaleBarLength = 0.100;				// size of scale bar in m
	static final double scaleBarInc = 0.100;		// increment for scale bar adjustment
		// ***** painting options ***
	static boolean curFileLoaded = false;				// flag indicates if the current file is loaded in memory
	static boolean paintScreen = true;				// flag to indicate actual drawing
	static boolean drawScaleBar = true;				// flag to indicate scale bar drawing
	static boolean drawTime = true;					// flag to indicate that time is shown in picture
	static boolean drawActinHelix = true;				// flag to indicate fancy helix drawing
	static boolean drawEachMonomer = true;			// flag to draw each monomer as a circle
	static boolean drawHelicalLines = true;				// flag to draw helix lines
	static double helixBlobDiam = 0.0054;			// diameter to use in drawing blobs
	static final double helixAngleInc = Math.PI + helixAngPerActin;  // angle increment in radians
	static final double helixXInc = lengthPerActin;		// x increment per line segment
	static final double helicalBlobOffset = (fActinDiameter-helixBlobDiam)/2;	// offset from center line
	static final boolean showDragEllipses = false;		// flag to show, or not, the equivalent ellipses used for drag
	static final boolean showHydrolysisStages = true;	// flag to paint hydrolyzed sections of actin
	static final boolean showARPLocations = true;		// flag to paint ARP2/3 locations on the filament
	static final boolean showADFLocations = true;		// flag to paint ADF/Cofilin locations on the filament
	static boolean rotateZoomAngle = false;			// flag for rotation of zoomPanel dispAng
	static boolean rotateAtEnd = true;				// flag for rotation at end of run
	static final double viewAngleStep = 2.0*Math.PI/180;	// increment in dispAng
	static boolean usePinCam = false;				// flag for pinhole camera viewing
	static final double plusCapDiam = fActinDiameter;	// size of plus cap
	static final double minusCapDiam = fActinDiameter; 	// size of minus cap
	static boolean trackListeria = false;				// flag for tracking listeria in painting
	static boolean reportNumTethered = false;			// whether or not to print number of rods tethered
	static boolean lisCircleOn = false;					// draw a circle around the listeria
	static boolean autoZoomIn = false;					// zoom in each frame
	static boolean autoZoomOut = false;					// zoom out each frame
		// ***** rapid plotting options *****
	static final int pointsToPlot = 20000;				// # of points to plot
		// ***** variable for plane sections ****
	static Pt3D normalToPlane = new Pt3D (0,0,1);		// normal to plane(s) used for section painting
	static Pt3D pointInPlane = new Pt3D (10,10,10);		// point in the plane 
		// ***** colors *****
	//** black background colors
	
	static final Color backgroundColor = Color.black;
	static final Color borderColor = Color.lightGray;
	static final Color ATPActinColor = new Color ((float)1.0,(float)1.0,(float)100.0/255);
	static final Color ADPPiActinColor = Color.orange;
	static final Color ADPActinColor = Color.red;
	static final Color cappedActinColor = Color.gray;
	//static final Color activeARPColor = new Color ((float)150.0/255,(float)200.0/255,(float)100.0/255);
	static final Color activeARPColor = Color.lightGray;
	static final Color inactiveARPColor = Color.gray;
	static final Color adfColor = Color.darkGray;
	static final Color listeriaColor = Color.white;
	static final Color plusCapColor = Color.white;
	//static final Color minusCapColor = new Color ((float)0.5,(float)0.5,(float)0.75);
	static final Color minusCapColor = Color.lightGray;
	static final Color tetherColor = Color.blue;
	static final Color boundActAColor = Color.blue;
	static final Color scaleBarColor = Color.white;
	static final Color timeColor = Color.white;
	static final Color lisCircleColor = Color.cyan;
	static final Color actinOutlineColor = Color.black;
	static final Color trackPathColor = Color.white;
	
	
	//** gray background colors
	/*
	static final Color backgroundColor = Color.lightGray;
	static final Color borderColor = Color.black;
	static final Color ATPActinColor = new Color ((float)1.0,(float)200.0/255,(float)50.0/255);
	static final Color ADPPiActinColor = Color.orange;
	static final Color ADPActinColor = Color.red;
	static final Color cappedActinColor = Color.gray;
	static final Color activeARPColor = new Color ((float)150.0/255,(float)200.0/255,(float)100.0/255);
	static final Color inactiveARPColor = Color.gray;
	static final Color adfColor = Color.green;
	static final Color listeriaColor = Color.darkGray;
	static final Color plusCapColor = Color.white;
	static final Color minusCapColor = new Color ((float)0.5,(float)0.5,(float)0.75);
	static final Color tetherColor = Color.blue;
	static final Color boundActAColor = Color.blue;
	static final Color scaleBarColor = Color.black;
	static final Color timeColor = Color.black;
	static final Color lisCircleColor = Color.cyan;
	static final Color actinOutlineColor = Color.black;
	*/
	
	//** dendritic figure from Pollard match colors
	/*
	static final Color backgroundColor = new Color((float)254.0/255,(float)240.0/255,(float)200.0/255);
	static final Color borderColor = Color.black;
	static final Color ATPActinColor = new Color ((float)1.0,(float)1.0,(float)100.0/255);
	static final Color ADPPiActinColor = Color.orange;
	static final Color ADPActinColor = Color.red;
	static final Color cappedActinColor = Color.gray;
	static final Color activeARPColor = new Color ((float)150.0/255,(float)200.0/255,(float)100.0/255);
	static final Color inactiveARPColor = Color.gray;
	static final Color adfColor = Color.darkGray;
	static final Color plusCapColor = Color.lightGray;
	static final Color minusCapColor = new Color ((float)0.5,(float)0.5,(float)0.75);
	static final Color showGridDefaultColor = Color.white;
	static final Color unboundActAColor = new Color ((float)0.50,(float)1.0,(float)0.25);
	static final Color boundActAColor = new Color ((float)0.45,(float)0.40,(float)0.85);
	static final Color listeriaColor = Color.black;
	static final Color zoomBoxColor = Color.lightGray;
	static final Color tetherColor = Color.blue;
	static final Color scaleBarColor = Color.black;
	static final Color timeColor = Color.black;
	static final Color lisCircleColor = Color.cyan;
	static final Color actinOutlineColor = Color.black;
	*/
	
		// **** plot colors ****
	static final Color plotBackColor = Color.black;
	
	//**** Current file info ***
	static Pt3D curLisPos = new Pt3D();
	
	//**** Font options
	static final Font controlFont = new Font(null,Font.PLAIN,9);
	static final Font listFont = new Font(null,Font.PLAIN,7);
	static final Font axisFont = new Font(null,Font.PLAIN,12);
	static final Font bigFont = new Font(null,Font.PLAIN,20);
	static final Font timeFont = new Font ("TimesRoman", Font.PLAIN, 11);
	static final Font scaleBarFont = new Font ("TimesRoman", Font.PLAIN, 11);

	
	//**** MOVIE OPTIONS ****
	static int framesToSkip = 1;				// sets frames to play, 1=all frames, 2=every other, etc
	static File movieName = null;				// the movie to play
	static String fileType = "qk";				// the type of movie to play... either "qk" or "raw"
	static boolean makeRawMovie = false;		// flag for making RAW movie from QK files
	static boolean makePNGMovie = false;		// flag for making JPEG frames from QK files
	
	public static void calculateCurrentTime (String mvFrame) {
		String numericPart = mvFrame.substring(mvFrame.length()-12, mvFrame.length()-3);
		Integer intSteps = Integer.valueOf(numericPart);
		Env.curTime = Env.deltaT*intSteps.intValue();
	}
	
	public static void allDataColumnsOn (int colCt) {
		dataColumnsOn = new boolean [colCt];
		for (int i=0;i<colCt;i++) { dataColumnsOn[i] = true; }
	}
	
	public static void addPathPt (Pt3D newPt) {
		Pt3D tempPt = new Pt3D(newPt);
		pathPts[pathPtCt] = tempPt;
		pathPtCt++;
	}
}