package rocketBugsRender;
/*
	HistoPlotPanel ... JPanel for plotting HistogramPlus
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import java.text.*;
import java.io.*;
import org.jibble.epsgraphics.*;

public class HistoPlotPanel extends JPanel {
	HistoPlotter parent;
	static BufferedImage theImage;
	static Graphics2D thePalate;
	static EpsGraphics2D epsG;
	Dimension curSize;
	double xMin,xMax,yMin,yMax;
	int xPix, yPix;
	double xPixFac, yPixFac;
	
	// painting options
	int [] insets = new int [] {20,10,5,20};
	Color backColor = Env.backgroundColor;
	Color boxColor = Env.backgroundColor;
	Color ticColor = Env.listeriaColor;
	Color histColor = Color.cyan;//Env.ATPActinColor;
	Color nameColor = Env.listeriaColor;
	int ticSpace = 5;
	int ticLabelSpace = 2;	// label every # of tics
	int yPixelPad = 5;
	static DecimalFormat intervalFormat = new DecimalFormat("#0.00;#0.00");
	static Font labelFont = new Font(null,Font.PLAIN,10);
	static Font nameFont = new Font(null,Font.PLAIN,14);




	public HistoPlotPanel (HistoPlotter pFrame) {
		talkln ("making HistoPlotPanel");
		parent = pFrame;
		talkln ("done!");
	}
	
	public void initialize (Dimension curSize) {
		this.curSize = curSize;
		xPix = curSize.width-insets[0]-insets[1];
		yPix = curSize.height-insets[2]-insets[3];
		setSize(curSize);
		theImage = (BufferedImage)createImage(curSize.width,curSize.height);
		thePalate = (Graphics2D)theImage.getGraphics();
		thePalate.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		thePalate.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		clearImage();
	}
	
	public void autoScale (HistogramPlus hist) {
		yMin = 0;		// for now, minimum zero assumed
		yMax = 0;
		for (int i=0;i<hist.binCt;i++) {
			if (hist.binSums[i] > yMax) { yMax = hist.binSums[i]; }
		}
		
		xMin = 1e32;
		xMax = -1e32;
		for (int i=0;i<hist.binCt;i++) {
			if (hist.binIntervals[i] > xMax) { xMax = hist.binIntervals[i]; }
			if (hist.binIntervals[i] < xMin) { xMin = hist.binIntervals[i]; }
		}
	}
	
	public void autoScale (HistogramPlus [] hists) {
		yMin = 0;		// for now, minimum zero assumed
		yMax = 0;
		for (int j=0;j<hists.length;j++) {
			for (int i=0;i<hists[j].binCt;i++) {
				if (hists[j].binSums[i] > yMax) { yMax = hists[j].binSums[i]; }
			}
		}
		
		xMin = 1e32;
		xMax = -1e32;
		for (int i=0;i<hists[0].binCt;i++) {
			if (hists[0].binIntervals[i] > xMax) { xMax = hists[0].binIntervals[i]; }
			if (hists[0].binIntervals[i] < xMin) { xMin = hists[0].binIntervals[i]; }
		}
		talkln ("yMax = " + String.valueOf(yMax));
	}
	
	public void setYScale (double scaleFac) {
		yMax = scaleFac;
	}
	
	public void setPixFactors () {
		xPixFac = (xMax-xMin)/(double)xPix;
		yPixFac = (yMax-yMin)/(double)(yPix-yPixelPad);
		//talkln ("xPix = " + String.valueOf(xPix) + " ; range = " + String.valueOf(xMax-xMin));
		//talkln ("yPix = " + String.valueOf(yPix) + " ; range = " + String.valueOf(yMax-yMin));
	}
	
	public void showName (String nameString) {
		thePalate.setColor(nameColor);
		thePalate.setFont(nameFont);
		int xName = curSize.width - insets[1] - 100;
		int yName = insets[2] + 20;
		thePalate.drawString(nameString,xName,yName);
	}
	
	public void plotHist (HistogramPlus hist) {
		setPixFactors();
		// plot pixels loop
		int curY = 0,curX = 0,lastY = 0,lastX = 0;
		int ticCt = ticSpace;
		int ticLabelCt = ticLabelSpace;
		for (int i=0;i<hist.binCt;i++) {
			curY = (int)(insets[2] + yPix - (hist.binSums[i] - yMin)/yPixFac);
			curX = (int)(insets[0] + (hist.binIntervals[i] - xMin)/xPixFac);
			//talkln ("pix " + String.valueOf(i) + " : " + String.valueOf(curX) + "," + String.valueOf(curY));
			if (i != 0) {
				plotHistPoint(lastX,lastY,curX,curY);
			}
			
			if ((ticCt == ticSpace) | (i == hist.binCt-1)) {
				boolean printLabel = false;
				if ((ticLabelCt == ticLabelSpace) | (i == hist.binCt-1)) {
					printLabel = true;
					ticLabelCt = 0;
				}
					
				plotHistTic(curX, hist.binIntervals[i],printLabel);
				ticCt = 0;
				ticLabelCt++;
			}
			lastX = curX;
			lastY = curY;
			ticCt++;
		}
		repaint();
	}
	
	private void plotHistPoint (int x1,int y1,int x2,int y2) {
		thePalate.setColor(histColor);
		thePalate.drawLine(x1,y1,x2,y2);
	}
	
	private void plotHistTic (int x1, double value, boolean printVal) {
		int y1 = yPix + insets[2];
		int y2 = y1 + 5;
		thePalate.setColor(ticColor);
		thePalate.drawLine(x1,y1,x1,y2);
		if (printVal) {
			int xText = x1 - 10;
			int yText = y2 + 10;
			String label = String.valueOf(intervalFormat.format(value));
			thePalate.setFont(labelFont);
			thePalate.drawString(label,xText,yText);
		}
	}
	
	
	public void update(Graphics g) {
		paint(g);
	}
	
	public void paint (Graphics g) {
		g.drawImage(theImage,0,0,null);
	}
	
	public void eraseGraphBox () {
		thePalate.setColor(backColor);
		thePalate.setStroke(new BasicStroke(2.0f));
		thePalate.drawRect(insets[0],insets[2],xPix,yPix);
		thePalate.setStroke(new BasicStroke());
	}
	
	public void showGraphBox () {
		thePalate.setColor(boxColor);
		thePalate.setStroke(new BasicStroke(2.0f));
		thePalate.drawRect(insets[0],insets[2],xPix,yPix);
		thePalate.setStroke(new BasicStroke());
	}
	
	public void clearImage() {
		setBackground(Color.black);
		thePalate.setColor(backColor);
		thePalate.fillRect(0,0,curSize.width,curSize.height);
		showGraphBox();
	}
	
	private static void talkln (String info) {
		System.out.println(info);
	}
	
	private static void talk (String info) {
		System.out.println(info);
	}
}