package rocketBugsRender;
/*
	LineColumn ... extension of DataColumn where the data is represented by some number
	of lines
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;
import java.text.*;

public class LineColumn extends DataColumn {
	DataColumn parCol;
	Line [] lines;
	int lineCt = 0;
	int lineArrayInc = 100;
	boolean distinctLines = true;
	boolean singleCol = false;
	int [] curLineRange = new int[2];
	DecimalFormat pauseFormat = new DecimalFormat ("#0.0#; -#0.0#");
	
	public LineColumn (int rows, String name, DataSet set) {
		super(rows, name, set);
		lines = new Line[rows];
		
	}
	
	public LineColumn (int rows) {
		super(rows);
		lines = new Line[rows];
		
	}
	
	public LineColumn () {
		super();
	}
	
	public void setParentCol (DataColumn parCol) {
		this.parCol = parCol;
	}
	
	public void nullLinks() {
		parCol = null;
		for (int i=0;i<lineCt;i++) { lines[i] = null; }
		lines = null;
		curLineRange = null;
		pauseFormat = null;
		super.nullLinks();
	}
	
	public int getPixColCt () {
		if ((distinctLines) & (!singleCol)) { return lineCt; }
		return 1;
	}
	
	public Pix2DColumn [] getPixColumns () {
		// overrides method from DataColumn to provide overlapping lines
		if (!onY) { return null; }
		if (!distinctLines) { return super.getPixColumns(); }
		if (singleCol) {
			Pix2DColumn [] pixA = new Pix2DColumn[1];
			int ptCt=0;
			for (int i=0;i<lineCt;i++) { ptCt += lines[i].ptCt; }
			pixA[0] = new Pix2DColumn(ptCt,this);
			pixA[0].setName(name);
			for (int i=0;i<lineCt;i++) {
			// INCOMPLETE
			}
			return pixA;
		} else {
			Pix2DColumn [] pixA = new Pix2DColumn [lineCt];
			for (int i=0;i<lineCt;i++) {
				if ((lines[i].show) & (lineInBounds(lines[i]))) {
					pixA[i] = lineToPixs(i);
					pixA[i].setName(name);
					if (lines[i] instanceof Plateau) {
						Plateau curP = (Plateau)lines[i];
						pixA[i].annotated = true;
						String pauseString = "t=" + String.valueOf(pauseFormat.format(curP.pauseTime*1e3)) + "ms";
						String stepString = "s=" + String.valueOf(pauseFormat.format(curP.nextStepSize*1e3)) + "nm";
						pixA[i].annotation = pauseString + " ; " + stepString;
					}
				} else {
					pixA[i] = null;
				}
			}
			return pixA;
		}
	}
	
	public boolean lineInBounds(Line ckLine) {
		if ((ckLine.start > curRowRange[0]) & (ckLine.start < curRowRange[1])) { return true; }	// starts in range
		if ((ckLine.stop > curRowRange[0]) & (ckLine.stop < curRowRange[1])) { return true; }	// stops in range
		if ((ckLine.start < curRowRange[0]) & (ckLine.stop > curRowRange[1])) { return true; }	// spans range
		return false;
	}
	
	public void setScale (Dimension pixels, int [] rowRange) {
		if (parCol.onY) {
			// fix range to that from which this line data derives
			this.pixels = parCol.pixels;
			curRowRange[0] = parCol.curRowRange[0];
			curRowRange[1] = parCol.curRowRange[1];
			pProps.min = parCol.pProps.min;
			pProps.max = parCol.pProps.max;
			yPixFactor = parCol.yPixFactor;
			xPixFactor = parCol.xPixFactor;
			yPixOffset = parCol.yPixOffset;
			axis.turnOff();
			return;
		}
		
		super.setScale(pixels, rowRange);
	}
			
	
	public void setCurLineRange () {
		curLineRange[0] = 0;	// clear old values
		curLineRange[1] = lineCt;
		boolean gotStart = false;
		boolean gotStop = false;
		for (int i=0;i<lineCt;i++) {
			Line curL = lines[i];
			if (!gotStart) {
				if ((curL.start <= curRowRange[0]) & (curL.stop >= curRowRange[0])) {
					curLineRange[0] = i;
					gotStart = true;
				}
			}
			if (!gotStop) {
				if ((curL.start <= curRowRange[1]) & (curL.stop >= curRowRange[1])) {
					curLineRange[1] = i;
					gotStop = true;
				}
			}
		}
	}
	
	public void allToNaN () {
		for (int i=0;i<rows;i++) {
			data[i] = Double.NaN;
		}
	}
	
	public void linesToPoints () {
		allToNaN();	// helps when lines are sparse
		double [] pts;
		for (int i=0;i<lineCt;i++) {
			int start = lines[i].start;
			pts = lines[i].getAllVals();
			for (int j=0;j<pts.length;j++) {
				data[j+start] = pts[j];
			}
		}
	}
	
	public Pix2DColumn lineToPixs (int i) {
		double [] pts = lines[i].getAllVals();
		pixsCol = new Pix2DColumn(pts.length,this);
		for (int j=0;j<pts.length;j++) {
			pixsCol.pixs[j] = getPix2DForPoint (lines[i].start + j, pts[j]);
		}
		return pixsCol;
	}
	
	public void ptsToLines () {}
	
	public void clearLines () {
		lineCt = 0;
	}
	
	public void addLine (Line newLine) {
		if (lineCt>=lines.length) {
			incLineArray();
		}
		lines[lineCt] = newLine;
		lineCt++;
	}
	
	public void removeLine (int rmIndex) {
		lines[rmIndex] = null;		
		for (int i=rmIndex+1;i<lineCt;i++) {	// shift all lines.. must keep order intact
			lines[i-1] = lines[i];
		}
		lines[lineCt-1] = null;
		lineCt += -1;
	}
	
	public void incLineArray() {
		Line[] temp = lines;
		lines = new Line[lineCt+100];
		for (int i=0;i<lineCt;i++) {
			lines[i] = temp[i];
		}
	}
			
}