package rocketBugsRender;
/*
	Listeria... lightweight version for painting
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class Listeria extends Player {
	static double length = Env.listeriaLength;			// length of listeria in micrometers
	static double cylDiam = Env.listeriaCylDiam;			//  diameter of cylindrical baterial section
	static double capRad = cylDiam/2;	// might as well do this division once.... radius of cap hemisphere.
	
	Pt3D rec1, rec2;		// locations of cap centers

	// some declarations related to painting the listeria
	int numCircs = 4;
	int numPts = 10;
	int numLines = 15;
	int numCapPts = 10;
	Pt3D [][] capPaintPts = new Pt3D [numCircs+1][numPts+1];	// store points in body axes for painting spherical caps
	Pt3D [] cylPaintPts = new Pt3D [numLines+1];				// ... for painting cylindrical section
	Pt3D [][] capPts = new Pt3D [numLines+1][numCapPts+1];

	public Listeria () {
		makePaintPoints();		// create painting features
	}
	
	public void initializeFromFileInfo (Pt3D rec1, Pt3D rec2, Pt3D angle) {
		this.rec1 = rec1;
		this.rec2 = rec2;
		this.angle = angle;
		this.coord.add(0.5,rec1,rec2);
		this.initialize();
	}
	
	public void initializeFromFileInfo (double length, double cylDiam, Pt3D rec1, Pt3D rec2, Pt3D angle) {
		Listeria.length = length;
		Listeria.cylDiam = cylDiam;
		Listeria.capRad = cylDiam/2;
		this.rec1 = rec1;
		this.rec2 = rec2;
		this.angle = angle;
		this.coord.add(0.5,rec1,rec2);
		this.initialize();
	}
	
	public static Listeria InitializeFromFileInfo (Pt3D rec1, Pt3D rec2, Pt3D angle) {
		Listeria listeria = new Listeria ();
		listeria.rec1 = rec1;
		listeria.rec2 = rec2;
		listeria.angle = angle;
		listeria.coord.add(0.5,rec1,rec2);
		listeria.initialize();
		return listeria;
	}
	
	public static Listeria InitializeFromFileInfo (double length, double cylDiam, Pt3D rec1, Pt3D rec2, Pt3D angle) {
		Listeria listeria = new Listeria ();
		Listeria.length = length;
		Listeria.cylDiam = cylDiam;
		Listeria.capRad = cylDiam/2;
		listeria.rec1 = rec1;
		listeria.rec2 = rec2;
		listeria.angle = angle;
		listeria.coord.add(0.5,rec1,rec2);
		listeria.initialize();
		return listeria;
	}
	
	public void initialize () {
		if ((Env.trackPath) & (!Env.paused)) { Env.addPathPt(coord); }
		// calculate cos and sin of Euler angles only once during each time step
		cosAng.cosine(angle);
		sinAng.sine(angle);
		// find the transformation matrices at this time step
		transMat ();
		// find the unit vector that defines the orientation of the rod
		uVec.xToX(this, xUnitVector);
		t1Vec.xToX(this, yUnitVector); 
		t2Vec.cross(uVec, t1Vec);
	}
	
	public void makePaintPoints () {
		double capRadSqrd = Math.pow(capRad,2);
		// Concentric circle pts
		double xRad, yRad, zRad, yLim;
		for (int i = 0; i <=numCircs; i++) {
			xRad = i*capRad/numCircs;
			if (i == numCircs) { xRad += -capRad/(4*numCircs); }
			yLim = Math.sqrt(capRadSqrd - Math.pow(xRad,2));
			for (int j = 0; j <= numPts; j++) {
				yRad = j*yLim/numPts;
				zRad = Math.sqrt(capRadSqrd - Math.pow(yRad,2) - Math.pow(xRad,2));
				if (Double.isNaN(zRad)) { zRad = 0; };
				capPaintPts[i][j] = new Pt3D (xRad,yRad,zRad);
			}
		}
			
		// Cylindrical section pts
		double theta;
		double thetaInc = 2*Math.PI/numLines;
		for (int i = 0; i <= numLines; i++) {
			theta = i*thetaInc;
			yRad = (cylDiam/2)*Math.cos(theta);
			zRad = (cylDiam/2)*Math.sin(theta);
			cylPaintPts[i] = new Pt3D (0,yRad,zRad);
		}
		
		// Spherical caps
		double phi;
		double phiInc = Math.PI/(2*numCapPts);
		for (int i = 0; i <= numLines; i++) {
			theta = i*thetaInc;
			for (int j = 0; j <= numCapPts; j++) {
				phi = j*phiInc;
				xRad = capRad*Math.cos(phi);
				yRad = capRad*Math.sin(phi)*Math.cos(theta);
				zRad = capRad*Math.sin(phi)*Math.sin(theta);
				capPts [i][j] = new Pt3D(xRad,yRad,zRad);
			}
		}			
	}
				
	public void showYourself (Graphics g, double scale, PinCam cam) {
		g.setColor(Env.listeriaColor);
		// note: xoffset, yoffset are non-zero only for the zoom window
		// Concentric circle pts
		Pt3D curPt;
		Pt3D [] pts = new Pt3D[4];
		Pixel [][] segPix = new Pixel [8][2];
		double zSign;
		// initialize arrays of Pt3D and Pixel
		for (int i = 0; i < 4; i++ ) { pts[i] = new Pt3D(); }
		for (int i = 0; i < 8; i ++) { segPix[i][0] = new Pixel();  segPix[i][1] = new Pixel(); }
		
		// The cylindrical section of the Listeria
		Pt3D dR = new Pt3D();
		Pt3D ptA = new Pt3D(); Pt3D ptB = new Pt3D();
		Pixel pixA = new Pixel(); Pixel pixB = new Pixel();
		for (int j = 0; j <= numLines; j++) {
			curPt = cylPaintPts[j];
			dR.xToX(this, curPt);
			ptA.add(rec1, dR);
			ptB.add(rec2, dR);
			if (Env.usePinCam) {
				pixA = cam.ptToPix(ptA,scale);
				pixB = cam.ptToPix(ptB,scale);
			} else {
				pixA.ptToPix(ptA,scale,cam.offset,cam.e);
				pixB.ptToPix(ptB,scale,cam.offset,cam.e);
			}
			if ((pixA.show) | (pixB.show)) {Pixel.drawLine(g,pixA,pixB);}
		}
		
		// spherical cap of the Listeria
		Pt3D [][] cPts = new Pt3D [2][2];
		Pixel [][] cPix = new Pixel [2][2];
		for (int i = 0; i < 2; i++) { cPts[i][0] = new Pt3D(); cPts[i][1] = new Pt3D(); }
		for (int i = 0; i < 2; i++) { cPix[i][0] = new Pixel(); cPix[i][1] = new Pixel(); }
		
		for (int i = 0; i <= numLines; i++) {
			dR.xToX(this, capPts[i][0]);
			cPts[0][0].add(rec2, dR);
			dR.xToX(this, new Pt3D(-capPts[i][0].x, capPts[i][0].y, capPts[i][0].z));
			cPts[1][0].add(rec1, dR);
			if (Env.usePinCam) {
				cPix[0][0] = cam.ptToPix(cPts[0][0],scale);
				cPix[1][0] = cam.ptToPix(cPts[1][0],scale);
			} else {
				cPix[0][0].ptToPix(cPts[0][0],scale,cam.offset,cam.e);
				cPix[1][0].ptToPix(cPts[1][0],scale,cam.offset,cam.e);
			}
			for (int j = 1; j <= numCapPts; j++) {
				dR.xToX(this, capPts[i][j]);
				cPts[0][1].add(rec2, dR);
				dR.xToX(this, new Pt3D(-capPts[i][j].x, capPts[i][j].y, capPts[i][j].z));
				cPts[1][1].add(rec1, dR);
				if (Env.usePinCam) {
					cPix[0][1] = cam.ptToPix(cPts[0][1],scale);
					cPix[1][1] = cam.ptToPix(cPts[1][1],scale);
				} else {
					cPix[0][1].ptToPix(cPts[0][1],scale,cam.offset,cam.e);
					cPix[1][1].ptToPix(cPts[1][1],scale,cam.offset,cam.e);
				}
				if ((cPix[0][0].show) | (cPix[0][1].show)) {Pixel.drawLine(g, cPix[0][0], cPix[0][1]);}
				if ((cPix[1][0].show) | (cPix[1][1].show)) {Pixel.drawLine(g, cPix[1][0], cPix[1][1]);}
				cPix[0][0].copy(cPix[0][1]);
				cPix[1][0].copy(cPix[1][1]);
			}
		}
		
		if (Env.lisCircleOn) {
			Pixel cirCen = new Pixel();
			g.setColor(Env.lisCircleColor);
			Pt3D lisCoord = Pt3D.Add(0.5,rec1,rec2);	// the center
			cirCen.ptToPix(lisCoord,scale,cam.offset,cam.e);
			int circRad = (int)(.70*length*scale);
			g.drawOval(cirCen.x-circRad,cirCen.y-circRad,2*circRad,2*circRad);
		}
	}

}