package rocketBugsRender;
/*
	The Player Class... the parent class of all the things that move (and some that don't)  in our model
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.lang.Math;
import java.awt.*;

public class Player {

	static Player [] playerList = new Player[Env.maxPlayers];	// to store players when positioning
	static int playerCount = 0;						// int to count players stored above
	static final Pt3D xUnitVector = new Pt3D(1,0,0);	// unit vector along body-fixed x-axis
	static final Pt3D yUnitVector = new Pt3D(0,1,0);	// unit vector along body-fixed y-axis
	static final Pt3D zUnitVector = new Pt3D(0,0,1);	// unit vector along body-fixed z-axis
	static final Pt3D zeroVec = new Pt3D();	// just a zero Pt3D

	Color playerColor;
	Pt3D coord = new Pt3D();  			// the coordinate point for the CM 
	Pt3D angle = new Pt3D();			// the angle values for the player
	Pt3D uVec = new Pt3D();		// the unit vector that describes the orientation of the player
	Pt3D t1Vec = new Pt3D();		// the first transverse vector for this body... used by filaments for branch orientation
	Pt3D t2Vec = new Pt3D();		// second transverse vector for this body
	Pt3D cosAng = new Pt3D();	// cosines of Euler angles {psi,theta,phi}
	Pt3D sinAng = new Pt3D();	// sines of Euler angles {psi,theta,phi}
	double [][] transXTox = new double [3][3];	// transformation matrix from fixed to body-fixed frame
	double [][] transxToX = new double [3][3];	// the inverse transformation... body-fixed to fixed

	public Player () {
	}
	
	public void showYourself (Graphics g, double scale, PinCam cam) {}
	
	public void transMat () {
		transXTox [0][0] = cosAng.x*cosAng.y;
		transXTox [0][1] = sinAng.x*cosAng.y;
		transXTox [0][2] = -sinAng.y;
		transXTox [1][0] = -sinAng.x*cosAng.z+cosAng.x*sinAng.y*sinAng.z;
		transXTox [1][1] = cosAng.x*cosAng.z+sinAng.x*sinAng.y*sinAng.z;
		transXTox [1][2] = cosAng.y*sinAng.z;
		transXTox [2][0] = sinAng.x*sinAng.z+cosAng.x*sinAng.y*cosAng.z;
		transXTox [2][1] = -cosAng.x*sinAng.z+sinAng.x*sinAng.y*cosAng.z;
		transXTox [2][2] = cosAng.y*cosAng.z;
		// the inverse transformation is found as the transpose of this matrix
		// since the direction cosine matrix is orthogonal.
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				transxToX [i][j] = transXTox [j][i];
			}
		}
	}
	

	public void talk (String info) {
		System.out.print(info);
	}
	
	public void talkln (String info) {
		System.out.println(info);
	}
	
}