package rocketBugsRender;
/* 
	PlotPanel.... the panel in which to plot and analyze
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import javax.swing.*;
import java.text.*;
import java.io.*;
import org.jibble.epsgraphics.*;

public class PlotPanel extends JPanel {
	RocketBugsRender MP;		// a reference back to the parent JFrame of this panel
	static Image theImage;
	static Graphics2D thePalate;
	static EpsGraphics2D epsG;
	static Dimension plotDim = new Dimension();	// dimensions for complete graph
	static Dimension lineDim = new Dimension();	// dimensions for lines
	static double [] plotScale;
	static int pixInc = 1;		// number of plot points to skip
	static int labelSpacing = 8; // pixel spacing between y-axes
	static int labelStart = 3;	// right side of first axis
	static boolean flatAxis = false;	// flag for splayed axis columns
	int [] offsets = new int [] {150,30,5,50};	// buffers around plot: x-Left, x-Right, y-Top, y-Bottom
	Pt3D offset = new Pt3D ();	// store {x,y,z} offsets for the zoom window
	Pt3D zoomCen = new Pt3D ();	// store zoom center
	int [] cursorPos = {0,0};	// store initial cursor position for dispAng and Z changes
	PinCam plotCam;
	DecimalFormat scaleBarFormat = new DecimalFormat ("#0.0#; -#0.0#");
	DecimalFormat showXFormat = new DecimalFormat ("#0.0#;-#0.0#");
	DecimalFormat axisFormat = new DecimalFormat ("00.000#E0");
	Color backColor = Env.plotBackColor; // default background color
	Color boxColor = Color.white;	// default box color... set to x-axis color
	BasicStroke defaultStroke = new BasicStroke();
	
	// for saving eps file
	static boolean epsOut = false;
	
	// current axes references and positions
	Axis [] curYAxes = null;
	int [] yAxesPositions = null;
	
	public PlotPanel (RocketBugsRender MP) {
		this.MP = MP;
	}
	
	public void initialize (Dimension plotDim) {
		setDimensions(plotDim);
		setSize(plotDim);
		theImage = createImage(plotDim.width,plotDim.height);
		thePalate = (Graphics2D)theImage.getGraphics();
		thePalate.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
		thePalate.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		clearImage();
	}
	
	public void openEpsFile () {
	
		try {
			String epsFileName = FileOps.getFileToSave("Save EPS file as...", MP);
			File epsFile = new File(epsFileName);
			String title = "EPS grab";
			epsG = new EpsGraphics2D(title,epsFile,0,0,plotDim.width,plotDim.height);
		} catch (IOException ioe) { talkln ("Problem creating EpsGraphics2D in PlotPanel.java"); }
		thePalate = epsG;	// switch reference to eps file
	}
	
	public void closeEpsFile () {
		try {
			epsG.close();
		} catch (IOException ioe) { talkln ("Problem closing epsG in PlotPanel.java"); }
		epsOut = false;
		thePalate = (Graphics2D)theImage.getGraphics();	// switch reference back to screen image
	}
		
	
	public void setDimensions (Dimension plotDim) {
		this.plotDim = plotDim;
		setLineDim();
	}
	
	public void setLineDim () {
		lineDim.width = plotDim.width - offsets[0] - offsets[1];
		lineDim.height = plotDim.height - offsets[2] - offsets[3];
	}
	
	private boolean pixInBounds (int x, int y) {
		// check x
		if ((x<offsets[0]) | (x>offsets[0]+lineDim.width)) { return false; }
		// check y
		if ((y<offsets[2]) | (y>offsets[2]+lineDim.height)) { return false; }
		return true;
	}
	
	private boolean mousePtInBounds (int x, int y) {
		// check x
		if ((x<offsets[0]) | (x>offsets[0]+lineDim.width)) { return false; }
		// check y
		if ((y<offsets[2]) | (y>offsets[2]+lineDim.height)) { return false; }
		return true;
	}
	
	public String getXStringFromPixel(Point mousePt, DataColumn xColumn) {
		if (xColumn == null) { return null; }
		if (!mousePtInBounds(mousePt.x,mousePt.y)) { return null; }
		double slope = (xColumn.pProps.max - xColumn.pProps.min)/lineDim.width;	// double value change per pixel
		double val = xColumn.pProps.min + slope*(mousePt.x-offsets[0]);
		String valStr = String.valueOf(showXFormat.format(val));
		return valStr;
	}
	
	public void setPixInc (Pix2DColumn pixCol) {
		if (pixCol.length > 2*Env.pointsToPlot) {
			pixInc = (int)(pixCol.length/Env.pointsToPlot);
		} else {
			pixInc = 1;
		}
	}
	
	public void plotPixs(Pix2DColumn pixCol) {
		thePalate.setStroke(pixCol.pProps.pStroke);
		thePalate.setColor(pixCol.pProps.color);
		if (pixCol.annotated) {
			Pix2D startTextPix = pixCol.pixs[pixCol.length-1];
			int xStart = startTextPix.x + offsets[0] + 20;
			int yStart = startTextPix.y + offsets[2] - pixCol.pProps.pixOrigin;
			thePalate.drawString(pixCol.annotation,xStart,yStart);
		}
		if ((pixCol.pProps.useLines) | (pixCol.pProps.useDashedLines)) {
			thePalate.setStroke(pixCol.pProps.pStroke);
			for (int i=0;i<pixCol.length-1;i+=pixInc) {
				int xStart = pixCol.pixs[i].x+offsets[0];
				int yStart = pixCol.pixs[i].y+offsets[2]-pixCol.pProps.pixOrigin;
				int xStop = pixCol.pixs[i+1].x+offsets[0];
				int yStop = pixCol.pixs[i+1].y+offsets[2]-pixCol.pProps.pixOrigin;
				if ((pixInBounds(xStart,yStart)) & (pixInBounds(xStop,yStop))) {
					thePalate.drawLine(xStart,yStart,xStop,yStop);
				}
			}
		}
		if (pixCol.pProps.usePoints) {
			for (int i=0;i<pixCol.length;i+=pixInc) {
				int width = 2;
				int height = 2;
				int xStart = pixCol.pixs[i].x+offsets[0]-width/2;
				int yStart = pixCol.pixs[i].y+offsets[2]-pixCol.pProps.pixOrigin-height/2;
				if (pixInBounds(xStart,yStart)) {
					thePalate.drawOval(xStart,yStart,width,height);
				}
			}
		}
		if (pixCol.pProps.useBars) {
			if (pixCol.pProps.zeroBars) {
				for (int i=0;i<pixCol.length-1;i+=pixInc) {
					int xStart = pixCol.pixs[i].x+offsets[0];
					int yStart = pixCol.pixs[i].y+offsets[2]-pixCol.pProps.pixOrigin;
					int xStop = xStart;
					int yStop = plotDim.height - offsets [3] - pixCol.parentCol.axis.yZero - pixCol.pProps.pixOrigin;
					//int yStop = plotDim.height-offsets[3]-pixCol.pixOrigin;
					if ((pixInBounds(xStart,yStart)) & (pixInBounds(xStop,yStop))) {
						thePalate.drawLine(xStart,yStart,xStop,yStop);
					}
				}
			} else {
			 	for (int i=0;i<pixCol.length-1;i+=pixInc) {
					int xStart = pixCol.pixs[i].x+offsets[0];
					int yStart = pixCol.pixs[i].y+offsets[2]-pixCol.pProps.pixOrigin;
					int xStop = xStart;
					int yStop = plotDim.height-offsets[3]-pixCol.pProps.pixOrigin;
					if ((pixInBounds(xStart,yStart)) & (pixInBounds(xStop,yStop))) {
						thePalate.drawLine(xStart,yStart,xStop,yStop);
					}
				}
			}
		}
		thePalate.setStroke(defaultStroke);
	}
	
	public void plotPixs(Pix2DColumn [] setPix) {
		for (int i=0;i<setPix.length;i++) {
			if (setPix[i] != null) {
				setPixInc(setPix[i]);
				plotPixs(setPix[i]); 
			}
		}
	}
	
	public void eraseYAxis (Axis yAx, int curPosition) {
		thePalate.setColor(backColor);
		int axWidth = yAx.labelWidth + yAx.ticWidth;
		int xStart = offsets[0]-curPosition-axWidth;
		int yStart = offsets[2];
		thePalate.fillRect(xStart,yStart,axWidth+labelStart,lineDim.height+10);
	}
	
	public void plotYAxis (Axis yAx, int curPosition) {
		if (yAx.pProps.showAxis) {
			thePalate.setFont(Env.axisFont);
			Tic [] tics = yAx.getTics();
			for (int i=0;i<tics.length;i++) {
				int xStart = offsets[0]-tics[i].width-curPosition; 
				if (flatAxis) { xStart = offsets[0] - tics[i].width; } // all axes in one column
				int xStop = xStart + tics[i].width;
				int yStart = lineDim.height - tics[i].loc + offsets[2] - yAx.pProps.pixOrigin;
				int yStop = yStart;
				//thePalate.setColor(boxColor);
				//thePalate.drawString(">", xStart-4,yStart+4);
				//thePalate.drawLine(xStart,yStart,xStop,yStop);
				int stringLen = tics[i].label.length();
				thePalate.setColor(yAx.color);
				thePalate.drawString(tics[i].label + yAx.pProps.unitsString + " >", xStart-yAx.labelWidth, yStart+4);
			}
		}
	}
	
	public void plotYAxis (AxisSet yAxes) {
		// compact array to eliminate possible NULL axes
		int axesCt = 0;
		for (int i=0;i<yAxes.axCt;i++) {
			if (yAxes.axes[i] != null) { axesCt++; }
		}
		curYAxes = new Axis [axesCt];
		int ct = 0;
		for (int i=0;i<yAxes.axCt;i++) {
			if (yAxes.axes[i] != null) { 
				curYAxes[ct] = yAxes.axes[i];
				ct++;
			}
		}
		if (axesCt > 0) {
			// establish positions of y-axes...
			yAxesPositions = new int [axesCt];
			yAxesPositions[0] = labelStart;
			for (int i=1;i<axesCt;i++) {
					yAxesPositions[i] = yAxesPositions[i-1] + curYAxes[i-1].labelWidth + labelSpacing;
			}
			// draw
			for (int i=0;i<axesCt;i++) { plotYAxis(curYAxes[i],yAxesPositions[i]); }
		}
	}
	
	
	public void plotXAxis (Axis xAx) {
		thePalate.setColor(xAx.color);
		boxColor = xAx.color;
		thePalate.setFont(Env.axisFont);
		Tic [] tics = xAx.getTics();
		for (int i=0;i<tics.length;i++) {
			int xStart = offsets[0]+tics[i].loc;
			int xStop = xStart;
			int yStart = lineDim.height + offsets[2];
			int yStop = yStart+tics[i].width;
			thePalate.drawLine(xStart,yStart,xStop,yStop);
			thePalate.drawString(tics[i].label,xStart-16,yStop+10);
		}
	}
	
	public void update(Graphics g) {
		paint(g);
	}
	
	public void paint (Graphics g) {
		g.drawImage(theImage,0,0,null);
	}
	
	public void eraseGraphBox () {
		thePalate.setColor(backColor);
		thePalate.setStroke(new BasicStroke(2.0f));
		thePalate.drawRect(offsets[0],offsets[2],lineDim.width-1,lineDim.height-1);
		//thePalate.drawRect(offsets[0]+1,offsets[2]+1,lineDim.width-2,lineDim.height-2);
		thePalate.setStroke(new BasicStroke());
	}
	
	public void showGraphBox () {
		thePalate.setColor(boxColor);
		thePalate.setStroke(new BasicStroke(2.0f));
		thePalate.drawRect(offsets[0],offsets[2],lineDim.width-1,lineDim.height-1);
		//thePalate.drawRect(offsets[0]+1,offsets[2]+1,lineDim.width-3,lineDim.height-3);
		thePalate.setStroke(new BasicStroke());
	}
	
	public void clearImage() {
		setBackground(Color.black);
		thePalate.setColor(backColor);
		thePalate.fillRect(0,0,plotDim.width,plotDim.height);
		showGraphBox();
	}

	
	public void showScaleBar () {
		/*thePalate.setColor(Env.scaleBarColor);
		thePalate.setFont(new Font ("TimesRoman", Font.PLAIN, 9));
		int endBarSize = 3;
		Pixel pixA = new Pixel(20,20);
		Pixel pixB = new Pixel(pixA.x + (int)(Env.scaleBarLength*winScale), pixA.y);
		Pixel.drawLine(thePalate, pixA, pixB);
		Pixel barA = new Pixel(pixA.x, pixA.y-endBarSize);
		Pixel barB = new Pixel(pixA.x, pixA.y+endBarSize);
		Pixel.drawLine(thePalate, barA, barB);
		barA = new Pixel(pixB.x, pixB.y-endBarSize);
		barB = new Pixel(pixB.x, pixB.y+endBarSize);
		Pixel.drawLine(thePalate, barA, barB);
		// text for scale bar
		String scaleString;
		if (Env.scaleBarLength >= 1.0) {
			scaleString = String.valueOf(scaleBarFormat.format(Env.scaleBarLength)) + " m";	// scale bar string in micrometers
		} else {
			scaleString = String.valueOf(scaleBarFormat.format(Env.scaleBarLength*1000)) + " nm";	// scale bar string in nanometers
		}
		thePalate.drawString(scaleString, (int)((pixA.x + pixB.x)/2-12), pixA.y-2*endBarSize);*/
	}
	
	
	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
}