package rocketBugsRender;
/* 
	PlotProps ... plotting properties for a given DataColumn
*/

/*  <RocketBugsRender - for rendering and graphing the output of Zoom (and other programs)>
    Copyright (C) <2007>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.awt.*;
import java.awt.event.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.text.*;
import java.util.*;

public class PlotProps extends JFrame implements ChangeListener, ActionListener, WindowListener, ComponentListener, KeyListener, MouseListener, MouseMotionListener, AdjustmentListener {
	DataColumn parentDataCol;
	JPanel titlePanel;
	JPanel continuePanel;
	JPanel plotOptPanel;
	JComboBox plotTypeBox, textTypeBox;
	JLabel titleLabel,minLabel,maxLabel,unitsLabel,unitsMultLabel,textTypeLabel,decPtLabel;
	JTextField minField,maxField,strokeField,unitsField,unitsMultField,decPtField,pixOrigField,pixFracField;
	JCheckBox customLimitsBox, holdRangeBox, showAxisBox;
	JButton plotButton;		// a link to button that will update the plot
	boolean fieldError = false;
	
	// plotting properties to carry
	String name = "";
	String unitsString = "";	// add units label to tics
	double unitMult = 1.0;
	boolean showAxis = true;	// whether axis value and tics should be shown
	double min, max;
	boolean customLimits = false;
	boolean holdRange;
	Tic [] tics;
	int numTics = 3;
	int ticWidth = 5;
	int labelWidth = 0;		// number of pixels required to fit the label string
	int pixOrigin = 0;		// pixels off the x-axis to begin plotting
	double pixFrac = 1;		// fraction of pixel height to use for plotting
	boolean fixedTic = true;
	double fixedTicInc = 0.0054;
	// plot type and color
	Color color = Color.white;	// DEFAULT color to plot with
	boolean useLines = true;	// DEFAULT data pts connected by lines
	boolean useDashedLines = false; // dashed lines
	boolean usePoints = false;	// just pts
	boolean useBars = false;	/// bars to the origin
	boolean zeroBars = false;	// bars start at zero.... good for showing sign changes
	boolean zeroMin = false;	// bars at zero 2
	// stroke definitions
	float strokeWidth = (float)1.0;
	float [] dashPattern = {4,4};
	float dashOffset = 2;
	Stroke pStroke = new BasicStroke();
	
	// text formats
	int decPtCt = 2;
	DecimalFormat expFormat = new DecimalFormat ("0.00E0");
	DecimalFormat decFormat = new DecimalFormat ("0.00");
	DecimalFormat intFormat = new DecimalFormat ("0");
	DecimalFormat textFormat = expFormat;

	public PlotProps (DataColumn parentDataCol) {
		this.parentDataCol = parentDataCol;
		
		makeTitlePanel();
		makeContinuePanel();
		makePlotOptPanel();
		
		setSize(200,300);
		
		addKeyListener(this);
		addComponentListener(this);
		addMouseListener(this);
		addMouseMotionListener(this);
		addWindowListener(this);
		
		initialize();
	}
	
	public void initialize () {
		getContentPane().removeAll();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add (titlePanel, BorderLayout.NORTH);
		getContentPane().add (plotOptPanel, BorderLayout.CENTER);
		getContentPane().add (continuePanel, BorderLayout.SOUTH);
	
	}
	
	public void adjustPixOrigin (int newPixOrigin) {
		pixOrigin = newPixOrigin;
	}
	
	public void adjustPixFrac(double newHeightFrac) {
		pixFrac = newHeightFrac;
	}
	
	public int getPixOrigin() {
		return 1;
	}
			
	
	public void getProps (JButton pB) {
		minField.setText(String.valueOf(min));
		maxField.setText(String.valueOf(max));
		strokeField.setText(String.valueOf(strokeWidth));
		unitsField.setText(unitsString);
		unitsMultField.setText(String.valueOf(unitMult));
		decPtField.setText(String.valueOf(decPtCt));
		pixOrigField.setText(String.valueOf(pixOrigin));
		pixFracField.setText(String.valueOf(pixFrac));
		plotButton = pB;
		show();
	}
	
	public void propsFromGuiCancel () {
		customLimitsBox.setSelected(customLimits);
		enableLimitsFields(customLimits);
		holdRangeBox.setSelected(holdRange);
	}
	
	public void propsFromGui () {
		customLimits = customLimitsBox.isSelected();
		holdRange = holdRangeBox.isSelected();
		showAxis = showAxisBox.isSelected();
		
		if (customLimits) {   // parse limits from text fields
			Double DD = Double.valueOf(minField.getText());
			min = DD.doubleValue();
			if (Double.isNaN(min)) { 
				minField.setForeground(Color.red);
				fieldError = true;
			} else {
				minField.setForeground(Color.black);
			}
			
			DD = Double.valueOf(maxField.getText());
			max = DD.doubleValue();
			if (Double.isNaN(min)) { 
				maxField.setForeground(Color.red);
				fieldError = true;
			} else {
				maxField.setForeground(Color.black);
			}
		}
		
		Double DE = Double.valueOf(strokeField.getText());
		strokeWidth = (float)DE.doubleValue();
		
		if (showAxis) {
			unitsString = unitsField.getText();
			
			Double DF = Double.valueOf(unitsMultField.getText());
			unitMult = DF.doubleValue();
			
			Double DG = Double.valueOf(decPtField.getText());
			decPtCt = (int) DG.doubleValue();
		}
		
		
		Double DH = Double.valueOf(pixOrigField.getText());
		pixOrigin = (int) DH.doubleValue();
		
		Double DJ = Double.valueOf(pixFracField.getText());
		pixFrac = DJ.doubleValue();
		
		// set style of plot
		setPlotType ();
		
		// create the drawing stroke to use
		createPlotStroke();
		
		// set text style on axis
		setTextType ();
		setTextDecimals ();
	}
	
	private void setPlotType () {
		int pointType = 0;
		int lineType = 1;
		int dashType = 2;
		int barType = 3;
		int barTypeShowZero = 4;
		int barTypeZeroMin = 5;
		
		useLines = false;
		useDashedLines = false;
		usePoints = false;
		useBars = false;
		zeroBars = false;
		zeroMin = false;
		int pType = plotTypeBox.getSelectedIndex();
		if (pType == lineType ) { useLines = true; }
		if (pType == dashType) { useDashedLines = true; }
		if (pType == pointType) { usePoints = true; }
		if (pType == barType) { useBars = true; }
		if (pType == barTypeZeroMin) {
			useBars = true;
			zeroMin = true;
		}
		if (pType == barTypeShowZero) {
			useBars = true;
			zeroBars = true;
		}
	}
	
	private void setTextType () {
		int expType = 0;
		int decType = 1;
		int intType = 2;
		
		int tType = textTypeBox.getSelectedIndex();
		if (tType == decType) { textFormat = decFormat; }
		if (tType == expType) { textFormat = expFormat; }
		if (tType == intType) { textFormat = intFormat; }
	}
	
	private void setTextDecimals () {
		String basicForm = "0.";
		for (int i=0;i<decPtCt;i++) {
			basicForm = basicForm + "0";
		}
		decFormat = new DecimalFormat(basicForm);
		expFormat = new DecimalFormat(basicForm + "E0");
	}
		
	
	public void createPlotStroke() {
		if (useDashedLines) {
			pStroke = new BasicStroke(strokeWidth, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0f, dashPattern, dashOffset);
		} else {
			pStroke = new BasicStroke(strokeWidth);
		}
	}
	
	private void enableLimitsFields(boolean on) {
		if (on) {
			minLabel.setEnabled(true);
			minField.setEnabled(true);
			maxLabel.setEnabled(true);
			maxField.setEnabled(true);
		} else {
			minLabel.setEnabled(false);
			minField.setEnabled(false);
			maxLabel.setEnabled(false);
			maxField.setEnabled(false);
		}
	}
	
	private void enableShowAxisFields(boolean on) {
		if (on) {
			unitsMultLabel.setEnabled(true);
			unitsMultField.setEnabled(true);
			unitsLabel.setEnabled(true);
			unitsField.setEnabled(true);
			decPtLabel.setEnabled(true);
			decPtField.setEnabled(true);
		} else {
			unitsMultLabel.setEnabled(false);
			unitsMultField.setEnabled(false);
			unitsLabel.setEnabled(false);
			unitsField.setEnabled(false);
			decPtLabel.setEnabled(false);
			decPtField.setEnabled(false);
		}
	}
	
	public void actionPerformed (ActionEvent e) {
		String act = e.getActionCommand();
		
		if (act.equals("Cancel")) {
			propsFromGuiCancel();
			hide();
		}
		
		if (act.equals("Apply")) {
			fieldError = false;
			propsFromGui();
			if (!fieldError) { plotButton.doClick(); }
		}
		
		if (act.equals("CustomLimits")) {
			enableLimitsFields(customLimitsBox.isSelected());
		}
		
		if (act.equals("ShowAxis")) {
			enableShowAxisFields(showAxisBox.isSelected());
		}
		
		
	}
	
	public void stateChanged(ChangeEvent e) {
        JSlider source = (JSlider)e.getSource();
    }
	
	public void adjustmentValueChanged (AdjustmentEvent e) {
	}
	
	
	public void windowActivated (WindowEvent e) {}
	public void windowClosing (WindowEvent e) {}
	public void windowDeactivated (WindowEvent e) {}
	public void windowDeiconified (WindowEvent e) {}
	public void windowIconified (WindowEvent e) {}
	public void windowOpened (WindowEvent e) {}
	public void windowClosed (WindowEvent e) {}
	
	public void keyReleased (KeyEvent e) {}
	public void keyTyped (KeyEvent e) {}
	public void keyPressed (KeyEvent e) {
		String act = e.getKeyText(e.getKeyCode());
	}
	
	public void mouseClicked (MouseEvent e) {}
	public void mouseEntered (MouseEvent e) {}
	public void mouseExited (MouseEvent e) {}
	public void mousePressed (MouseEvent e) {
		int xPos = e.getX();
		int yPos = e.getY();
	}
	
	public void mouseReleased (MouseEvent e) {}
	public void mouseDragged (MouseEvent e) {}
	public void mouseMoved (MouseEvent e) {}
	
	public void componentHidden (ComponentEvent e) {}
	public void componentShown (ComponentEvent e) {}
	public void componentMoved (ComponentEvent e) {}
	public void componentResized (ComponentEvent e) {}
	
	public void makeTitlePanel () {
		titlePanel = new JPanel();
		titleLabel = new JLabel("Plot options for " + parentDataCol.getName());
		titlePanel.add(titleLabel);
	}
	
	public void makePlotOptPanel() {
		plotOptPanel = new JPanel();
		
		JLabel dummyLabel = new JLabel("");
		
		JLabel plotTypeLabel = new JLabel("Plot Type:");
		plotTypeLabel.setFont(Env.controlFont);
		
		String [] plotTypeStrings = {"Points","Lines","Dashed Lines","Bars(min-->)","Bars(zero-->)","Bars(zero-->+)"};
		plotTypeBox = new JComboBox(plotTypeStrings);
		plotTypeBox.setSelectedIndex(1);
		plotTypeBox.setFont(Env.controlFont);
		plotTypeBox.setActionCommand("PlotType");
		plotTypeBox.setToolTipText("change plot type");
		plotTypeBox.addActionListener(this);
		
		// user modified plot limits
		customLimitsBox = new JCheckBox("Custom plot limits");
		customLimitsBox.setActionCommand("CustomLimits");
		customLimitsBox.setFont(Env.controlFont);
		customLimitsBox.setSelected(false);
		customLimitsBox.addActionListener(this);
		
		holdRangeBox = new JCheckBox("Hold range");
		holdRangeBox.setActionCommand("HoldRange");
		holdRangeBox.setFont(Env.controlFont);
		holdRangeBox.setSelected(false);
		holdRangeBox.addActionListener(this);
		
		// text field dimension
		Dimension textDim = new Dimension(50,20);
		
		minLabel = new JLabel("Min. axis value:");
		minLabel.setFont(Env.controlFont);
		minLabel.setEnabled(customLimits);

		minField = new JTextField(6);
		minField.setFont(Env.controlFont);
		minField.setPreferredSize(textDim);
		minField.setEnabled(customLimits);
		
		maxLabel = new JLabel("Max. axis value:");
		maxLabel.setFont(Env.controlFont);
		maxLabel.setEnabled(customLimits);
		
		maxField = new JTextField(6);
		maxField.setFont(Env.controlFont);
		maxField.setPreferredSize(textDim);
		maxField.setEnabled(customLimits);
		
		JLabel strokeLabel = new JLabel("Stroke width:");
		strokeLabel.setFont(Env.controlFont);
		
		strokeField = new JTextField(6);
		strokeField.setFont(Env.controlFont);
		strokeField.setPreferredSize(textDim);
		
		// show axis at all
		showAxisBox = new JCheckBox("Show axis");
		showAxisBox.setActionCommand("ShowAxis");
		showAxisBox.setFont(Env.controlFont);
		showAxisBox.setSelected(true);
		showAxisBox.addActionListener(this);
		
		unitsMultLabel = new JLabel ("Mulitply by");
		unitsMultLabel.setFont(Env.controlFont);
		unitsMultLabel.setEnabled(showAxis);
		
		unitsMultField = new JTextField(6);
		unitsMultField.setFont(Env.controlFont);
		unitsMultField.setPreferredSize(textDim);
		unitsMultField.setEnabled(showAxis);
		
		unitsLabel = new JLabel("Label with units");
		unitsLabel.setFont(Env.controlFont);
		unitsLabel.setEnabled(showAxis);
		
		unitsField = new JTextField(6);
		unitsField.setFont(Env.controlFont);
		unitsField.setPreferredSize(textDim);
		unitsField.setEnabled(showAxis);
		
		textTypeLabel = new JLabel("Text type:");
		textTypeLabel.setFont(Env.controlFont);
		textTypeLabel.setEnabled(showAxis);
		
		String [] textTypeStrings = {"exponent","decimal","integer"};
		textTypeBox = new JComboBox(textTypeStrings);
		textTypeBox.setSelectedIndex(0);
		textTypeBox.setFont(Env.controlFont);
		textTypeBox.setActionCommand("TextType");
		textTypeBox.setToolTipText("change text type");
		textTypeBox.addActionListener(this);
		textTypeBox.setEnabled(showAxis);
		
		decPtLabel = new JLabel("Decimals:");
		decPtLabel.setFont(Env.controlFont);
		decPtLabel.setEnabled(showAxis);
		
		decPtField = new JTextField(6);
		decPtField.setFont(Env.controlFont);
		decPtField.setPreferredSize(textDim);
		decPtField.setEnabled(showAxis);
		
		JLabel pixOrigLabel = new JLabel("Pix Offset:");
		pixOrigLabel.setFont(Env.controlFont);
		
		pixOrigField = new JTextField(6);
		pixOrigField.setFont(Env.controlFont);
		pixOrigField.setPreferredSize(textDim);
		
		JLabel pixFracLabel = new JLabel("Pix Fraction:");
		pixFracLabel.setFont(Env.controlFont);
		
		pixFracField = new JTextField(6);
		pixFracField.setFont(Env.controlFont);
		pixFracField.setPreferredSize(textDim);
		
		
		plotOptPanel.setLayout(new GridLayout(12,2));
		plotOptPanel.add(plotTypeLabel);
		plotOptPanel.add(plotTypeBox);
		plotOptPanel.add(customLimitsBox);
		plotOptPanel.add(holdRangeBox);
		plotOptPanel.add(minLabel);
		plotOptPanel.add(minField);
		plotOptPanel.add(maxLabel);
		plotOptPanel.add(maxField);
		plotOptPanel.add(strokeLabel);
		plotOptPanel.add(strokeField);
		plotOptPanel.add(showAxisBox);
		plotOptPanel.add(dummyLabel);
		plotOptPanel.add(unitsMultLabel);
		plotOptPanel.add(unitsMultField);
		plotOptPanel.add(unitsLabel);
		plotOptPanel.add(unitsField);
		plotOptPanel.add(textTypeLabel);
		plotOptPanel.add(textTypeBox);
		plotOptPanel.add(decPtLabel);
		plotOptPanel.add(decPtField);
		plotOptPanel.add(pixOrigLabel);
		plotOptPanel.add(pixOrigField);
		plotOptPanel.add(pixFracLabel);
		plotOptPanel.add(pixFracField);
	}

	public void makeContinuePanel () {
		continuePanel = new JPanel();
	
		JButton cancelB = new JButton ("Cancel");
		cancelB.setActionCommand("Cancel");
		cancelB.setToolTipText("Cancel Changes");
		cancelB.addActionListener(this);
		
		JButton setB = new JButton("Apply");
		setB.setActionCommand("Apply");
		setB.setToolTipText("Apply Changes");
		setB.addActionListener(this);
		
		continuePanel.setLayout(new FlowLayout());
		continuePanel.add(cancelB);
		continuePanel.add(setB);
	}

	private void talk (String info) {
		System.out.print(info);
	}
	
	private void talkln (String info) {
		System.out.println(info);
	}
	

}